"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("underscore");
var generateApprovalRequestDifferencesByRules_1 = require("./generateApprovalRequestDifferencesByRules");
var generateDifferences_1 = require("./generateDifferences");
var revertFields_1 = require("./revertFields");
var PLCARBatch = /** @class */ (function () {
    function PLCARBatch(rules, comparisons, user) {
        this.rules = rules;
        this.comparisons = comparisons;
        this.user = user;
    }
    PLCARBatch.prototype.shouldBeApproval = function (dtos) {
        return this.build(dtos).length > 0;
    };
    PLCARBatch.prototype.revert = function (dtos) {
        var _this = this;
        var diffs = this.build(dtos);
        _(dtos).map(function (dto) {
            var dtoDiffs = _(diffs).where({ sourceId: dto._id });
            revertFields_1.revertFields(dto, dtoDiffs);
            dto.children = _this.revert(dto.children);
        });
        return dtos;
    };
    PLCARBatch.prototype.build = function (dtos) {
        var user = this.findOwnerRules(this.rules, this.user);
        if (user.length > 0) {
            var differences = this.diffs(dtos);
            return generateApprovalRequestDifferencesByRules_1.generateApprovalRequestDifferencesByRules(this.rules, differences);
        }
        return [];
    };
    PLCARBatch.prototype.updateComparison = function (comparisons) {
        this.comparisons = comparisons;
    };
    PLCARBatch.prototype.findOwnerRules = function (rules, owner) {
        return _(rules)
            .chain()
            .filter(function (it) {
            if (it.byRole === "*") {
                return true;
            }
            return _(owner.teams).contains(it.byRole) || _(it.byUserIds).contains(owner._id);
        })
            .value();
    };
    PLCARBatch.prototype.diffs = function (dtos, comparisons) {
        var _this = this;
        if (comparisons === void 0) { comparisons = this.comparisons; }
        var comparisonIds = _(comparisons).pluck("_id");
        return _(dtos)
            .chain()
            .filter(function (dto) {
            return _(comparisonIds).contains(dto._id);
        })
            .map(function (dto) {
            var comparison = _(comparisons).findWhere({ _id: dto._id });
            return _this.diff(dto, comparison);
        })
            .flatten()
            .value();
    };
    PLCARBatch.prototype.diff = function (dto, comparison) {
        var diffs = generateDifferences_1.generateDifferences(comparison, dto);
        var childrenDiffs = this.diffs(dto.children, comparison.children);
        return _.union(diffs, childrenDiffs);
    };
    return PLCARBatch;
}());
exports.PLCARBatch = PLCARBatch;
//# sourceMappingURL=PLCARBatch.js.map