"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("underscore");
function generateApprovalRequestDifferencesByRules(rules, differences) {
    var resultDifferences = [];
    // if user both in team demo.admin && demo, and trigger condition is same except approver is different, then combine rules from: [
    // {byRole:"demo.admin", field:"status", from: "draft", shouldApproveByRole:"demo.manager", shouldApproveByUserIds: [111]},
    // {byRole:"demo", field:"status", from: "draft", shouldApproveByRole:"demo.manager", shouldApproveByUserIds: [222]}
    // ]
    // to
    // [{byRole:"demo.admin", field:"status", from: "draft", shouldApproveByRole:"demo.manager", shouldApproveByUserIds: [111, 222]}]
    var combineRules = _(rules)
        .chain()
        .map(function (rule, index) {
        var exceptSelfRules = _.without(rules, rules[index]);
        var hasSameRule = _.findWhere(exceptSelfRules, { field: rule["field"], from: rule["from"], to: rule["to"] });
        if (hasSameRule) {
            var id = _.union(rule["shouldApproveByUserIds"], hasSameRule["shouldApproveByUserIds"]);
            hasSameRule["shouldApproveByUserIds"] = id;
            return hasSameRule;
        }
        else {
            return rule;
        }
    })
        .uniq(function (combineRule) {
        return combineRule.field + combineRule.from + combineRule.to;
    })
        .value();
    _.each(combineRules, function (it) {
        var diff;
        if (it.from === "*" || it.to === "*") {
            diff = _(differences).findWhere({
                field: it.field,
            });
        }
        else {
            diff = _(differences).findWhere({
                field: it.field,
                before: it.from,
                after: it.to,
            });
        }
        if (diff) {
            if ((it.from === "*" || it.from === diff.before) && (it.to === "*" || it.to === diff.after)) {
                resultDifferences.push(_.extend(diff, {
                    approveByRole: it.shouldApproveByRole,
                    approveByUserIds: it.shouldApproveByUserIds,
                }));
            }
        }
    });
    return resultDifferences;
}
exports.generateApprovalRequestDifferencesByRules = generateApprovalRequestDifferencesByRules;
//# sourceMappingURL=generateApprovalRequestDifferencesByRules.js.map