import {ApprovalRequestRuleCollection} from "kaneoh-data-types/types/records/PlatformConfigRecord";
import {ApprovalRequestDifference, ApprovalRequestDifferences} from "kaneoh-data-types/types/records/ApprovalRequestRecord";
import * as _ from "underscore";

export function generateApprovalRequestDifferencesByRules(
    rules: ApprovalRequestRuleCollection, differences: ApprovalRequestDifferences
): ApprovalRequestDifferences {
    const resultDifferences = [] as ApprovalRequestDifferences;

    // if user both in team demo.admin && demo, and trigger condition is same except approver is different, then combine rules from: [
    // {byRole:"demo.admin", field:"status", from: "draft", shouldApproveByRole:"demo.manager", shouldApproveByUserIds: [111]},
    // {byRole:"demo", field:"status", from: "draft", shouldApproveByRole:"demo.manager", shouldApproveByUserIds: [222]}
    // ]
    // to
    // [{byRole:"demo.admin", field:"status", from: "draft", shouldApproveByRole:"demo.manager", shouldApproveByUserIds: [111, 222]}]
    const combineRules = _(rules)
        .chain()
        .map((rule, index) => {
            const exceptSelfRules = _.without(rules, rules[index]);
            const hasSameRule = _.findWhere(exceptSelfRules, {field : rule["field"], from : rule["from"], to : rule["to"]});
            if (hasSameRule) {
                const id = _.union(rule["shouldApproveByUserIds"], hasSameRule["shouldApproveByUserIds"]);
                hasSameRule["shouldApproveByUserIds"] = id;
                return hasSameRule;
            } else {
                return rule;
            }
        })
        .uniq((combineRule) => {
            return combineRule.field + combineRule.from + combineRule.to;
        })
        .value();

    _.each(combineRules, (it) => {
        let diff: ApprovalRequestDifference;
        if (it.from === "*" || it.to === "*") {
            diff = _(differences).findWhere({
                field: it.field,
            });
        } else {
            diff = _(differences).findWhere({
                field: it.field,
                before: it.from,
                after: it.to,
            });
        }

        if (diff) {
            if ((it.from === "*" || it.from === diff.before) && (it.to === "*" || it.to === diff.after)) {
                resultDifferences.push(
                    _.extend(diff,
                        {
                            approveByRole: it.shouldApproveByRole,
                            approveByUserIds: it.shouldApproveByUserIds,
                        }
                    )
                );
            }
        }
    });

    return resultDifferences;
}
