import {ApprovalRequestDifference} from "kaneoh-data-types/types/records/ApprovalRequestRecord";
import {lookup} from "../commons/lookup";
import * as _ from "underscore";
import * as DeepDiff from "deep-diff";
import IDiff = deepDiff.IDiff;

export function generateDifferences<T>(comparison: T, dto: T): Array<ApprovalRequestDifference> {
    return _(DeepDiff.diff(comparison, dto)).map((difference: IDiff) => {
        let fieldValue;
        const lastPath = _.last(difference.path);
        if (_.isNumber(lastPath)) {
            const lastValue = lookup(comparison, _.initial(difference.path));
            if (_.isArray(lastValue)) {
                fieldValue = _.initial(difference.path).join(".");
            }
        } else {
            fieldValue = difference.path.join(".");
        }

        let {lhs, rhs} = difference;
        if (fieldValue === "ownerIds") {
            lhs = comparison["ownerIds"];
            rhs = dto["ownerIds"];
        }

        return {
            field: fieldValue,
            before: lhs,
            after: rhs,
            sourceId: dto["_id"]
        };
    });
}
