import {
    CustomizedDefectSummaryColumn, CustomizedDefectSummaryDto,
    CustomizedDefectSummaryRow
} from "kaneoh-data-types/types/dtos/CustomizedDefectSummaryDto";
import {DefectRecord} from "kaneoh-data-types/types/records/DefectRecord";
import {DefectSummaryFilterDto, DefectSummaryFilterDtoItem} from "kaneoh-data-types/types/dtos/DefectSummaryFilterDto";
import * as Mingo from "mingo";
import {KFilterUtils} from "kaneoh-filter";
import * as _ from "underscore";

export function convertDefectsToCustomizedDefectSummaryDto(defects: Array<DefectRecord>,
                                                           filterDtos: Array<DefectSummaryFilterDto>): CustomizedDefectSummaryDto {

    const rows = _(filterDtos).map<CustomizedDefectSummaryRow>((dto: DefectSummaryFilterDto) => {
        const rowConditions = dto.row.conditions;
        const label = dto.row.label;
        const columns = _(dto.columns).map<CustomizedDefectSummaryColumn>((column: DefectSummaryFilterDtoItem) => {
            const allConditions = _(column.conditions).extend(rowConditions);
            const selector = KFilterUtils.buildSelector(allConditions);
            const query = new Mingo.Query(selector);
            const cursor = query.find(defects);
            return {
                label: column.label,
                value: cursor.count(),
                defects: cursor.all(),
            };
        });
        return {label, columns};
    });

    return {
        total: defects.length,
        title: "Defect Summary",
        rows,
    };
}
