import * as _ from "underscore";
import {DefectRecord} from "kaneoh-data-types/types/records/DefectRecord";
import {UserRecord} from "kaneoh-data-types/types/records/UserRecord";
import {isAdmin} from "../user/isAdmin";
import {DefectStatusAction} from "kaneoh-data-types/types/records/PlatformConfigRecord";

export function getDefectStatusesAvailableForUser(action: DefectStatusAction, defect: DefectRecord, user?: UserRecord): Array<string> {
    const userId = _.isEmpty(user) ? Meteor.userId() : user._id;

    if (!action) {
        return [];
    }

    let availableStatuses = _.chain([]);

    if (defect.submitterId === userId) {
        const statues = action.submitter;
        if (statues) {
            availableStatuses = availableStatuses.union(statues);
        }
    }

    if (defect.ownerId === userId) {
        const statues = action.owner;
        if (statues) {
            availableStatuses = availableStatuses.union(statues);
        }
    }

    if (isAdmin(user)) {
        const statues = action.submitter;
        if (statues) {
            availableStatuses = availableStatuses.union(statues);
        }
    }

    const statuses: Array<string> = availableStatuses.value() || [];

    if (!_.isUndefined(defect.status)) {
        statuses.unshift(defect.status);
    }

    return statuses;
}
